import utilities
import app
import animatedSprite16

BONUS_TYPE_SHOT_SPEED = 0
BONUS_TYPE_SHOT_DISTANCE = 0
BONUS_TYPE_SHOT_INTERVAL = 0
BONUS_OBJECT_ANIM_HOLD_DURATION = 15
BONUS_OBJECT_PICKUP_DISTANCE = 12.0

#
#
#
class BonusObjectManager:
	#
	#
	#
	def __init__(self, appInstance):
		self.app = appInstance
		
	#
	#
	#			
	def buildBonusObjectsList(self):
		self.listOfBonusObjects = []
				
		for row in range(0, utilities.TILE_ROWS) :
			for col in range (0, utilities.TILE_COLS) :	
				tileNr = self.app.levelTiles.getMidgroundTileNr(col, row)
												
				if tileNr == 180:
					bonusObjectType = BONUS_TYPE_SHOT_SPEED					
					spriteNrs = (60, 61, 62, 62, 61)
				elif tileNr == 181:				
					bonusObjectType = BONUS_TYPE_SHOT_DISTANCE			
					spriteNrs = (63, 64, 65, 65, 64)
				elif tileNr == 182:				
					bonusObjectType = BONUS_TYPE_SHOT_INTERVAL
					spriteNrs = (66, 67, 68, 68, 67)
				else:
					continue
					
				bonusObjectCoordinates = (col * utilities.SPRITE16_WIDTH, row * utilities.SPRITE16_HEIGHT)		
					
				newBonusObjectAnimatedSprite = animatedSprite16.AnimatedSprite16(bonusObjectCoordinates, spriteNrs, BONUS_OBJECT_ANIM_HOLD_DURATION, True)
				self.listOfBonusObjects.append([bonusObjectType, newBonusObjectAnimatedSprite])
				
				self.app.levelTiles.setMidgroundTileNr(col, row, -1)
		
	#
	#
	#		
	def prepareBonusObjects(self):
		self.buildBonusObjectsList()
		
	#
	#
	#		
	def updateBonusObjects(self):
		if self.app.isGameActive() == True:
			for index in range(len(self.listOfBonusObjects) - 1, -1, -1):
				position  = self.listOfBonusObjects[index][1].getPosition()
				x = position[0]
				y = position[1]	
	
				bonusObjectDistance = utilities.manhattenDistanceBetweenFloatCoordinates((self.app.playerPosition[0] + 12, self.app.playerPosition[1] + 12), (x + 8, y + 8))
				if bonusObjectDistance <= BONUS_OBJECT_PICKUP_DISTANCE:
					if self.listOfBonusObjects[index][0] == BONUS_TYPE_SHOT_SPEED:				
						self.app.playerShotSpeedModifier += 0.1
						if self.app.playerShotSpeedModifier > 5.0:
							self.app.playerShotSpeedModifier = 5.0						
					elif self.listOfBonusObjects[index][0] == BONUS_TYPE_SHOT_DISTANCE:	
						self.app.playerShotLifetimeModifier += 0.1
						if self.app.playerShotLifetimeModifier > 5.0:
							self.app.playerShotLifetimeModifier = 5.0						
					elif self.listOfBonusObjects[index][0] == BONUS_TYPE_SHOT_INTERVAL:			
						self.app.playerShotIntervalModifier += 0.1
						if self.app.playerShotIntervalModifier > 5.0:
							self.app.playerShotIntervalModifier = 5.0						
								
					self.app.addUpdwardSmokePuff((x, y))
					del self.listOfBonusObjects[index]

					self.app.soundFxManager.emitSoundEffect(self.app.soundFxManager.SFX_STAR_PICKUP_V3, 0)
					
								
		for bonusObjectCompound in self.listOfBonusObjects:
			bonusObjectCompound[1].update()
			
	#
	#
	#		
	def renderBonusObjects(self):
		for bonusObjectCompound in self.listOfBonusObjects:
			bonusObjectCompound[1].render()
	