import math

import utilities
import animatedSprite16
import app 

PILL_PICKUP_DISTANCE = 12.0
SCORE_PILL_NORMAL = 1
SCORE_PILL_ACTIVE = 5
PILL_ANIM_HOLD_DURATION = 15

#
#
#
class PillManager:
	#
	#
	#
	def __init__(self, appInstance):
		self.app = appInstance
		
	#
	#
	#			
	def buildPillList(self):
		firstPillTileNr = 160
		lastPillTileNr = 169
				
		for row in range(0, utilities.TILE_ROWS) :
			for col in range (0, utilities.TILE_COLS) :	
				tileNr = self.app.levelTiles.getMidgroundTileNr(col, row)
															
				if tileNr >= firstPillTileNr and tileNr <= lastPillTileNr:
					offset = tileNr - firstPillTileNr
					self.pillListPixelCoordinates[offset] = [col * utilities.SPRITE16_WIDTH, row * utilities.SPRITE16_HEIGHT]			
					self.app.levelTiles.setMidgroundTileNr(col, row, -1)
					
	#
	#
	#		
	def preparePills(self):
	
		self.pillListPixelCoordinates = [[-1, -1], [-1, -1], [-1, -1], [-1, -1], [-1, -1], [-1, -1], [-1, -1], [-1, -1], [-1, -1], [-1, -1]]
		self.buildPillList()
		
		self.nextPillIndex = 0		

		self.pickedUpPillIndexes = []

		pillSpriteNrs = (20, 21, 22, 23, 24, 25)

		self.animatedPillList = []		
		for pillIndex in range(0, len(self.pillListPixelCoordinates)):
			if pillIndex == self.nextPillIndex:
				doAnimate = True
			else:
				doAnimate = False
				 
			newPillAnimatedSprite = animatedSprite16.AnimatedSprite16(self.pillListPixelCoordinates[pillIndex], pillSpriteNrs, PILL_ANIM_HOLD_DURATION, doAnimate)
			self.animatedPillList.append(newPillAnimatedSprite)
			
	#
	#
	#		
	def updatePills(self):
		if self.app.isGameActive() == True:
			while self.nextPillIndex in self.pickedUpPillIndexes:
				self.nextPillIndex += 1	
						
			for index in range(self.nextPillIndex, len(self.pillListPixelCoordinates)):
				if index in self.pickedUpPillIndexes:
					continue
					
				x = self.pillListPixelCoordinates[index][0]
				y = self.pillListPixelCoordinates[index][1]	
	
				pillDistance = utilities.manhattenDistanceBetweenFloatCoordinates((self.app.playerPosition[0] + 12, self.app.playerPosition[1] + 12), (x + 8, y + 8))
				if pillDistance <= PILL_PICKUP_DISTANCE:							
					self.pickedUpPillIndexes.append(index)
					
					self.app.addUpdwardSmokePuff((x, y))
					self.app.soundFxManager.emitSoundEffect(self.app.soundFxManager.SFX_STAR_PICKUP_V3, 0)

					if index == self.nextPillIndex:
						self.app.addScore(SCORE_PILL_ACTIVE)
						
						while self.nextPillIndex in self.pickedUpPillIndexes:
							self.nextPillIndex += 1	
				
					else:
						self.app.addScore(SCORE_PILL_NORMAL)
						
					if 	self.nextPillIndex >= len(self.pillListPixelCoordinates):
						self.app.markMoveToNextLevel()	
					else:
						self.animatedPillList[self.nextPillIndex].setAnimationState(True)
		
		for index in range(self.nextPillIndex, len(self.pillListPixelCoordinates)):
			if index in self.pickedUpPillIndexes:
				continue
			
			self.animatedPillList[index].update()
												
	#
	#
	#		
	def renderPills(self):
		if self.nextPillIndex == len(self.pillListPixelCoordinates):
			return
			
		for index in range(self.nextPillIndex, len(self.pillListPixelCoordinates)):
			if index in self.pickedUpPillIndexes:
				continue
			
			self.animatedPillList[index].render()

	#
	#
	#
	def checkPlayerShotAgainstPills(self, sweepStartPosition, sweepEndPosition):
		shotVector = (sweepEndPosition[0] - sweepStartPosition[0], sweepEndPosition[0] - sweepStartPosition[0])
		shotVectorLength = math.sqrt(math.pow(shotVector[0], 2) + math.pow(shotVector[1], 2))
		
		shotVectorUnitLength = (shotVector[0] / shotVectorLength, shotVector[1] / shotVectorLength);
		
		for shotVectorStep in range(0, int(shotVectorLength), 16):
			targetCoordinate = (sweepStartPosition[0] + (shotVectorUnitLength[0] * shotVectorStep), sweepStartPosition[1] + (shotVectorUnitLength[1] * shotVectorStep)) 
			 
			for index in range(self.nextPillIndex, len(self.pillListPixelCoordinates)):
				if index in self.pickedUpPillIndexes:
					continue
	
				pillCenter = (self.pillListPixelCoordinates[index][0] + 8, self.pillListPixelCoordinates[index][1] + 8)					
			
				if utilities.distanceBetweenFloatCoordinates(targetCoordinate, pillCenter) < PILL_PICKUP_DISTANCE:
					self.pickedUpPillIndexes.append(index)
					
					while self.nextPillIndex in self.pickedUpPillIndexes:
						self.nextPillIndex += 1	
											
					if 	self.nextPillIndex >= len(self.pillListPixelCoordinates):
						self.app.markMoveToNextLevel()	
					else:
						self.animatedPillList[self.nextPillIndex].setAnimationState(True)
						
					return True	
		
		return False
