import os
import pygame
import sys
import random
import math

BLACK = (0, 0, 0)
DARKGREY = (52, 52, 52)
MIDGREY = (128, 128, 128)
LIGHTGREY = (204, 204, 204)

TILE_WIDTH = 16
TILE_HEIGHT = 16

TILE_COLS = 20
TILE_ROWS = 15

DISPLAY_WIDTH = 320
DISPLAY_HEIGHT = 240

CHAR_WIDTH = 8
CHAR_HEIGHT = 16

SPRITE8_WIDTH = 8
SPRITE8_HEIGHT = 8
SPRITE8_COLS = 40
SPRITE8_ROWS = 30
SPRITE8_IMG_PIXEL_WIDTH = 320
SPRITE8_IMG_PIXEL_HEIGHT = 240

SPRITE16_WIDTH = 16
SPRITE16_HEIGHT = 16
SPRITE16_COLS = 20
SPRITE16_ROWS = 15
SPRITE16_IMG_PIXEL_WIDTH = 320
SPRITE16_IMG_PIXEL_HEIGHT = 240

SPRITE24_WIDTH = 24
SPRITE24_HEIGHT = 24
SPRITE24_COLS = 10
SPRITE24_ROWS = 10
SPRITE24_IMG_PIXEL_WIDTH = 240
SPRITE24_IMG_PIXEL_HEIGHT = 240

TEXT_COLOR_WHITE = 0 
TEXT_COLOR_GREEN = 1 
TEXT_COLOR_BLUE = 2 
TEXT_COLOR_ORANGE = 3 

#
#
#
def prepareGraphicsSystem():
	global displayInfo, displaySurface, workSurface, resizedWorkSurface;
	global transparentColor
	global tilesSurface, signsSurface, levelBgSurface, levelFgSurface 
	global sprites8x8Surface, sprites16x16Surface, sprites24x24Surface
	global textString, numberString
	global calcW, calcH
	global offsetX, offsetY

	displayInfo = pygame.display.Info()

	# calculate aspect ratio correct scale 
	widthDiv = displayInfo.current_w / DISPLAY_WIDTH
	heightDiv = displayInfo.current_h / DISPLAY_HEIGHT
	
	if( widthDiv <= heightDiv):
		intW = int(widthDiv)
		calcW = intW * DISPLAY_WIDTH;
		calcH = intW * DISPLAY_HEIGHT
	else:
		intH = int(heightDiv)
		calcW = intH * DISPLAY_WIDTH;
		calcH = intH * DISPLAY_HEIGHT

	offsetX = int((displayInfo.current_w - calcW) / 2.0)
	offsetY = int((displayInfo.current_h - calcH) / 2.0)

	os.environ['SDL_VIDEO_WINDOW_POS'] = str(0) + "," + str(0) 

	#displayFlags = pygame.FULLSCREEN | pygame.DOUBLEBUF | pygame.HWSURFACE | pygame.NOFRAME
	#displayFlags = pygame.DOUBLEBUF | pygame.HWSURFACE | pygame.NOFRAME
	#displayFlags = pygame.DOUBLEBUF | pygame.NOFRAME
	displayFlags = pygame.NOFRAME # RetroPie
	#displayFlags = pygame.FULLSCREEN | pygame.NOFRAME # RetroPie (FS)
	
	displaySurface = pygame.display.set_mode((displayInfo.current_w, displayInfo.current_h), displayFlags)

	resizedWorkSurface = pygame.Surface((calcW, calcH))

	pygame.display.set_caption('Van Sangs Pain')
	
	pygame.mouse.set_visible(False)

	tilesSurface = pygame.image.load('VanSangsPain/images/tiles_16x16.png').convert()
	transparentColor = tilesSurface.get_at((0,0))
	tilesSurface.set_colorkey(transparentColor)

	workSurface = pygame.image.load('VanSangsPain/images/grid_16x16.png').convert()
	workSurface.fill(transparentColor, None, 0)

	textString = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ.-0123456789'
	numberString = '0123456789'

	signsSurface = pygame.image.load('VanSangsPain/images/signs.png').convert()
	signsSurface.set_colorkey(signsSurface.get_at((0,0)))

	levelBgSurface = pygame.image.load('VanSangsPain/images/grid_16x16.png').convert()

	levelFgSurface = pygame.image.load('VanSangsPain/images/grid_16x16.png').convert()
	levelFgSurface.set_colorkey(levelFgSurface.get_at((0,0)))

	sprites8x8Surface = pygame.image.load('VanSangsPain/images/sprites_8x8.png').convert()
	sprites8x8Surface.set_colorkey(sprites8x8Surface.get_at((0,0)))

	sprites16x16Surface = pygame.image.load('VanSangsPain/images/sprites_16x16.png').convert()
	sprites16x16Surface.set_colorkey(sprites16x16Surface.get_at((0,0)))

	sprites24x24Surface = pygame.image.load('VanSangsPain/images/sprites_24x24.png').convert()
	sprites24x24Surface.set_colorkey(sprites24x24Surface.get_at((0,0)))

#
#
#
def scaleDisplaySurface():
	pygame.transform.scale(workSurface, (calcW, calcH), resizedWorkSurface)
	displaySurface.blit(resizedWorkSurface, (offsetX, offsetY), None, 0)

#
#
#
def loadTitleBgToDisplay():
	tempSurface = pygame.image.load('VanSangsPain/images/title_screen.png').convert()
	workSurface.blit(tempSurface, workSurface.get_rect(), None, 0)
	
#
#
#
def loadHiscoreBgToDisplay():
	tempSurface = pygame.image.load('VanSangsPain/images/hiscore_screen.png').convert()
	workSurface.blit(tempSurface, workSurface.get_rect(), None, 0)

#
#
#
def loadSettingsBgToDisplay():
	tempSurface = pygame.image.load('VanSangsPain/images/settings_screen.png').convert()
	workSurface.blit(tempSurface, workSurface.get_rect(), None, 0)
			
#
#
#
def clearDisplaySurface():
	workSurface.fill(transparentColor)

#
#
#
def clearLevelBgSurface():
	levelBgSurface.fill(transparentColor)

#
#
#
def clearLevelFgSurface():
	levelFgSurface.fill(transparentColor)
		
#
#
#
def blitTileToDisplaySurface(targetPos, sourceRect):
	workSurface.blit(tilesSurface, targetPos, sourceRect)

#
#
#
def blitSignsToDisplaySurface(targetPos, sourceRect):
	workSurface.blit(signsSurface, targetPos, sourceRect)

#
#
#
def blitTileToLevelBgSurface(targetPos, sourceRect):
	levelBgSurface.blit(tilesSurface, targetPos, sourceRect)
	
#
#
#
def blitTileToLevelFgSurface(targetPos, sourceRect):
	levelFgSurface.blit(tilesSurface, targetPos, sourceRect)

#
#
#
def blitSprite8ToDisplaySurface(targetPos, sourceRect):
	workSurface.blit(sprites8x8Surface, targetPos, sourceRect)

#
#
#
def blitSprite16ToDisplaySurface(targetPos, sourceRect):
	workSurface.blit(sprites16x16Surface, targetPos, sourceRect)

#
#
#
def blitSprite24ToDisplaySurface(targetPos, sourceRect):
	workSurface.blit(sprites24x24Surface, targetPos, sourceRect)
	
#
#
#
def calcSourceRectFromSprite8Nr(spriteNr):
	y = spriteNr / SPRITE8_COLS
	x = spriteNr % SPRITE8_COLS
	return (x * SPRITE8_WIDTH, y * SPRITE8_HEIGHT, SPRITE8_WIDTH, SPRITE8_HEIGHT)

#
#
#
def calcSourceRectFromSprite16Nr(spriteNr):
	y = spriteNr / SPRITE16_COLS
	x = spriteNr % SPRITE16_COLS
	return (x * SPRITE16_WIDTH, y * SPRITE16_HEIGHT, SPRITE16_WIDTH, SPRITE16_HEIGHT)

#
#
#
def calcSourceRectFromSprite24Nr(spriteNr):
	y = spriteNr / SPRITE24_COLS
	x = spriteNr % SPRITE24_COLS
	return (x * SPRITE24_WIDTH, y * SPRITE24_HEIGHT, SPRITE24_WIDTH, SPRITE24_HEIGHT)
	
#
#
#
def showLevelBgSurface():
	workSurface.blit(levelBgSurface, workSurface.get_rect(), None, 0)

#
#
#
def showLevelFgSurface():
	workSurface.blit(levelFgSurface, workSurface.get_rect(), None, 0)

#
#
#
def drawTextToDisplay(text, (x, y), color = TEXT_COLOR_WHITE):
	if color == TEXT_COLOR_GREEN:
		textSourceY = 96
	elif color == TEXT_COLOR_BLUE:
		textSourceY = 112
	elif color == TEXT_COLOR_ORANGE:
		textSourceY = 128
	else: 
		textSourceY = 80
		
	textSourceX = 0 
		
	for textIndex in range (0, len(text)):
		for alphabetIndex in range (0, len(textString)):
			if text[textIndex] == textString[alphabetIndex]:
				textTargetOffsetX = x + (textIndex * CHAR_WIDTH)
				textSourceOffsetX = textSourceX + (alphabetIndex * CHAR_WIDTH)

				targetPos = (textTargetOffsetX, y)
				sourceRect = pygame.Rect(textSourceOffsetX, textSourceY, CHAR_WIDTH, CHAR_HEIGHT)
				workSurface.blit(signsSurface, targetPos, sourceRect)

#
#
#
def drawNumberToDisplay(numberText, (x, y), color = TEXT_COLOR_WHITE):
	pass
	
#
#
#
def drawLivesHeader():
	targetPos = (18, 2)
	sourceRect = pygame.Rect(0, 64, 48, 16)
	workSurface.blit(signsSurface, targetPos, sourceRect)

#
#
#
def drawLives(livesNumber):
	place1 = livesNumber / 10
	place0 = livesNumber % 10

	targetPos1 = (68, 2)
	sourceRect1 = pygame.Rect(224 + (place1 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos1, sourceRect1)
	
	targetPos0 = (68 + CHAR_WIDTH, 2)
	sourceRect0 = pygame.Rect(224 + (place0 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos0, sourceRect0)

#
#
#
def drawScoreHeader():
	targetPos = (196, 2)
	sourceRect = pygame.Rect(48, 64, 48, 16)
	workSurface.blit(signsSurface, targetPos, sourceRect)
	
#
#
#
def drawScore(scoreNumber):
	cutScoreNumber = scoreNumber
	place6 = cutScoreNumber / 1000000
	cutScoreNumber = cutScoreNumber - (place6 * 1000000)
	place5 = cutScoreNumber / 100000
	cutScoreNumber = cutScoreNumber - (place5 * 100000)
	place4 = cutScoreNumber / 10000
	cutScoreNumber = cutScoreNumber - (place4 * 10000)
	place3 = cutScoreNumber / 1000
	cutScoreNumber = cutScoreNumber - (place3 * 1000)
	place2 = cutScoreNumber / 100
	cutScoreNumber = cutScoreNumber - (place2 * 100)
	place1 = cutScoreNumber / 10
	place0 = cutScoreNumber % 10

	targetPos6 = (246, 2)
	sourceRect6 = pygame.Rect(224 + (place6 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos6, sourceRect6)
		
	targetPos5 = (246 + CHAR_WIDTH, 2)
	sourceRect5 = pygame.Rect(224 + (place5 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos5, sourceRect5)
		
	targetPos4 = (246 + (2 * CHAR_WIDTH), 2)
	sourceRect4 = pygame.Rect(224 + (place4 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos4, sourceRect4)
		
	targetPos3 = (246 + (3 * CHAR_WIDTH), 2)
	sourceRect3 = pygame.Rect(224 + (place3 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos3, sourceRect3)
		
	targetPos2 = (246 + (4 * CHAR_WIDTH), 2)
	sourceRect2 = pygame.Rect(224 + (place2 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos2, sourceRect2)
		
	targetPos1 = (246 + (5 * CHAR_WIDTH), 2)
	sourceRect1 = pygame.Rect(224 + (place1 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos1, sourceRect1)
		
	targetPos0 = (246 + (6 * CHAR_WIDTH), 2)
	sourceRect0 = pygame.Rect(224 + (place0 * CHAR_WIDTH), 80, CHAR_WIDTH, CHAR_HEIGHT)
	workSurface.blit(signsSurface, targetPos0, sourceRect0)
		
#
#
#
def drawGameOverSign():
	targetPos = (112, 104)
	sourceRect = pygame.Rect(0, 0, 96, 32)
	workSurface.blit(signsSurface, targetPos, sourceRect)
	
#
#
#
def drawGetReadySign(counter):
	offsetY = counter / 8.0
 
 	switch = counter % 15
	if(switch < 7):
		sourceOffsetSign = 0 
	else:
		sourceOffsetSign = 32
		 
	targetPos = (112, 104 - offsetY)
	sourceRect = pygame.Rect(96, 0 + sourceOffsetSign, 96, 32)
	workSurface.blit(signsSurface, targetPos, sourceRect)
	
#
#
#
def drawPausedSign(toggle):
	targetPos = (120, 112)
	sourceRect = pygame.Rect(192, 32 + (toggle * 16), 80, 16)
	workSurface.blit(signsSurface, targetPos, sourceRect)	
	
#
#
#
def drawLevelNr(levelNr):
	levelNrText = "LEVEL " + str(levelNr)
	
	textTargetX = 128
	textTargetY = 222
	
	textSourceX = 0 
	textSourceY = 80
		
	for textIndex in range (0, len(levelNrText)):
		for alphabetIndex in range (0, len(textString)):
			if levelNrText[textIndex] == textString[alphabetIndex]:
				textTargetOffsetX = textTargetX	+ (textIndex * CHAR_WIDTH)
				textSourceOffsetX = textSourceX + (alphabetIndex * CHAR_WIDTH)

				targetPos = (textTargetOffsetX, textTargetY)
				sourceRect = pygame.Rect(textSourceOffsetX, textSourceY, CHAR_WIDTH, CHAR_HEIGHT)
				workSurface.blit(signsSurface, targetPos, sourceRect)

#
#
#			
def manhattenDistanceBetweenFloatCoordinates(vecA, vecB):
	diffX = math.fabs(vecA[0] - vecB[0])
	diffY = math.fabs(vecA[1] - vecB[1])
	
	distance = diffX + diffY
	return distance
	

#
#
#			
def distanceBetweenFloatCoordinates(vecA, vecB):
	diffX = math.fabs(vecA[0] - vecB[0])
	diffY = math.fabs(vecA[1] - vecB[1])
	
	distance = math.sqrt(math.pow(diffX, 2.0) + math.pow(diffY, 2.0))
	return distance
	
#
#
#			
def buildSprite8SourceRectsFromSpriteNrList(spriteNrTuple):
	result = []
	
	for entry in spriteNrTuple:
		newRect = calcSourceRectFromSprite8Nr(entry)
		result.append(newRect)

	return result
		
#
#
#			
def buildSprite16SourceRectsFromSpriteNrList(spriteNrTuple):
	result = []
	
	for entry in spriteNrTuple:
		newRect = calcSourceRectFromSprite16Nr(entry)
		result.append(newRect)

	return result
		
#
#
#			
def buildSprite24SourceRectsFromSpriteNrList(spriteNrTuple):
	result = []
	
	for entry in spriteNrTuple:
		newRect = calcSourceRectFromSprite24Nr(entry)
		result.append(newRect)

	return result

#
#
#
def getTextString():
	return textString
		